#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat May 30 00:01:30 2020

@author: yves

    This file is part of Saqqarah.

    Saqqarah is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Saqqarah is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Saqqarah.  If not, see <https://www.gnu.org/licenses/>
"""

# expose directly the classes in the module
from .version import version, codename
from .tools import *
from .ParametersClass import Parameters, PrinterCoordinates
from .PyramidClass import Pyramid
from .PyramidPrinterClass import PyramidPrinter
from .PyramidPrinterTikzClass import PyramidPrinterTikz
from .PyramidPrinterImageClass import PyramidPrinterImage
from .PreferencesUIClass import PreferencesUI
from .SaqqarahUIClass import SaqqarahUI

